#
# Build OpenEmbedded-Core in different configurations
#

# Notes:
#
# Download all files before filling the mirror:
#  BB_GENERATE_MIRROR_TARBALLS = "1"
#  bitbake world -c fetchall
#

SUITE = oecore
BUILD_OPTIONS = noarchive nocheck

TOPDIR ?= ../..

-include local.mk
include $(TOPDIR)/lib/common.mk
include $(LIBDIR)/steps.mk

# Required commands to build openembedded-core + meta-linaro
PREREQS ?= \
	mkdir python patch make diffstat wget curl md5sum sha256sum tar \
	bzip2 gzip unzip sed awk gcc g++ chrpath

# All architectures to build (list of MACHINEs)
ARCHITECTURES ?= qemuarmv7a qemuarm qemumips qemuppc qemux86 qemux86-64

# Images to build
IMAGES ?= core-image-minimal core-image-sato qt4e-demo-image

# Where to pull the sources from
MIRROR ?= http://builds.linaro.org/toolchain/misc/mirror/openembedded

# The versions we're testing against
OECORE_VERSION ?= openembedded-core-1.3~20120910+git0de1827
BITBAKE_VERSION ?= bitbake-1.15+20120910+git57055d3
META_LINARO_VERSION ?= meta-linaro-0~20120831+git276abbe

# The flavor of the GCC to be used. Choices are:
#  '4.6%'       - GCC 4.6 based toolchain provided by OE-Core
#  '4.7%'       - GCC 4.7 based toolchain provided by OE-Core
#  'linaro-4.6' - Linaro GCC 4.6 based toolchain provided by meta-linaro
#  'linaro-4.7' - Linaro GCC 4.7 based toolchain provided by meta-linaro
GCC_FLAVOR_DEFAULT = linaro-4.7
GCC_FLAVOR ?= linaro-$(shell echo $(GCC_FLAVOR_DEFAULT) $(B) | sed -r "s/.+-([4-9]\.[6-9]).*/\1/" )

YCHECK_FAILED = (touch $(TOP)/$(STEP)-$*-failed.txt && false)

# bitbake doesn't like a + in the name so build at the top
XBUILD = $(SUITE)

# Build in parallel
SETTINGS = \
	'BB_NUMBER_THREADS="$(NPROCESSORS)"' \
	'PARALLEL_MAKE="$(PARALLEL)"'

# Downloads in a common location and only pull from the mirror
SETTINGS += \
	'DL_DIR = "$(TOP)/$(XBUILD)/downloads"' \
	'SOURCE_MIRROR_URL = "$(MIRROR)"' \
	'INHERIT += "own-mirrors"' \
	'DISABLED_MLH_BB_FETCH_PREMIRRORONLY = "1"'

# Delete the object files after successful compile 
# in order to save some space.  Don't use ccache due to the extra
# space and funny bugs like libgcc configure failing (ask michaelh).
SETTINGS += \
	'INHERIT += "rm_work"' \
	'CCACHE=""'

# Don't append the type of the libc to the name of TMPDIR
SETTINGS += \
	'TCLIBCAPPEND = ""'

# GCC flavor to be used
SETTINGS += \
	'GCCVERSION = "$(GCC_FLAVOR)"' \
	'GCCSDKVERSION = "$(GCC_FLAVOR)"'

SETTINGS_qemuarmv7a = \
	'DEFAULTTUNE = "armv7athf-neon"' \
	'ARM_INSTRUCTION_SET = "thumb"' \
	'IMAGE_ROOTFS_ALIGNMENT = "2048"'

# Calls the init script
INIT = $(XBUILD)/$(SUITE)/oe-init-build-env $(TOP)/$(XBUILD)

# Configure all architectures
configure: $(ARCHITECTURES:%=configure-%)

# Configure one architecture
configure-%:
	mkdir -p $(XBUILD)/$*
	. $(INIT)/$* > /dev/null
	sed -i "s;BBLAYERS.*;BBLAYERS = \"$(CURDIR)/$(XBUILD)/$(SUITE)/meta-linaro \\\;" $(XBUILD)/$*/conf/bblayers.conf
	sed -i "s/^MACHINE.*/MACHINE = \"$*\"/" $(XBUILD)/$*/conf/local.conf
	for i in $(SETTINGS) $(SETTINGS_$(*)); do \
		echo $$i >> $(XBUILD)/$*/conf/local.conf; \
	done

# Build all architectures
build: $(ARCHITECTURES:%=build-%)
	true

# Build one architecture
build-%:
	$(call PING_STEP,$@)
	. $(INIT)/$* && bitbake -c fetchall $(IMAGES) > $(TOP)/$(STEP)-$*.txt 2>&1 || $(YCHECK_FAILED)
	. $(INIT)/$* && bitbake -k $(IMAGES) >> $(TOP)/$(STEP)-$*.txt 2>&1 || $(YCHECK_FAILED)

# Test all architectures
testsuite: $(ARCHITECTURES:%=testsuite-%)
	true

# Test one architecture
testsuite-%:
	$(call PING_STEP,$@)
	for i in $(IMAGES); do \
		bash $(LIBDIR)/oecore-runqemu.sh "$*" \
			"$(TOP)/$(XBUILD)/$*/tmp/deploy/images/$$i-$*.ext3" \
			&> $(TOP)/$(STEP)-$*.txt || $(YCHECK_FAILED); \
	done

# Don't install anything as they'd end up being pushed to the build
# log directory
install:
	true

# Record the image and package sizes for all architectures
get-sizes: $(ARCHITECTURES:%=get-sizes-%)
	true

# Record the image and package sizes for one architecture
get-sizes-%:
	du -ab `find $(XBUILD)/$*/tmp/deploy/images -type f -name "*$(*)*"` > $(STEP)-$*.txt
	du -ab $(XBUILD)/$*/tmp/deploy/ipk/*/*ipk >> $(STEP)-$*.txt

ifdef GCC_VERSION
GCC_TARBALL = $(B)/$(GCC_VERSION).tar
endif

GCC_INC = $(XBUILD)/$(SUITE)/meta-linaro/recipes-devtools/gcc/gcc-$(GCC_FLAVOR).inc

# Fetch all the common files.
# Patch the GCC recipe if needed.  Set the directories based on the
# tarball directoreis.  Set the source URI to local.  Add the file
# checksums.
extract-top: $(B)/$(OECORE_VERSION).tar $(B)/$(META_LINARO_VERSION).tar $(B)/$(BITBAKE_VERSION).tar $(GCC_TARBALL)
	bash $(LIBDIR)/check-commands.sh $(PREREQS)
	rm -rf $(XBUILD)
	mkdir -p $(XBUILD)/$(SUITE)/{meta-linaro,bitbake} $(XBUILD)/downloads
	tar xaf $(B)/$(OECORE_VERSION).tar -C $(XBUILD)/$(SUITE) --strip-components=1
	tar xaf $(B)/$(BITBAKE_VERSION).tar -C $(XBUILD)/$(SUITE)/bitbake --strip-components=1
	tar xaf $(B)/$(META_LINARO_VERSION).tar -C $(XBUILD)/$(SUITE)/meta-linaro --strip-components=1
	# Apply any patches
	cat $(TOPDIR)/files/$(SUITE)/*.patch /dev/null | patch -p1 -d $(XBUILD)/$(SUITE)
	# Re-enable the vectoriser.  Done as sed instead of a patch for compatibility
	sed -i~ s#-fno-tree-vectorize## $(XBUILD)/$(SUITE)/meta/conf/machine/include/arm/arch-armv7a.inc
	# Test that the GCC tarball is good rather than failing later
	[ -z "$(GCC_TARBALL)" ] || tar taf $(GCC_TARBALL) > /dev/null
	# Patch the GCC version if required
	if [ -n "$(GCC_VERSION)" ]; then \
		dir=$$(basename $$(tar tf $(GCC_TARBALL) | head -n1)); \
		sed -i -r "s#(^S = .+)/gcc-.+\"#\1/$$dir\"#" $(GCC_INC); \
		sed -i -r "s#(^FILESPATH = .+)/gcc-.+}'#\1/$$dir'#" $(GCC_INC); \
		uri=file://$$(readlink -e $(GCC_TARBALL)); \
		sed -i -r "s#(^SRC_URI = .+)http.+.tar.bz2#\1$$uri#" $(GCC_INC); \
		sed -i -r "s/^SRC_URI\[/# SRC_URI[/" $(GCC_INC); \
		md5=$$(md5sum $(GCC_TARBALL) | awk '{print $$1;}'); \
		echo "SRC_URI[md5sum] = \"$$md5\"" >> $(GCC_INC); \
		sha256=$$(sha256sum $(GCC_TARBALL) | awk '{print $$1;}'); \
		echo "SRC_URI[sha256sum] = \"$$sha256\"" >> $(GCC_INC); \
	fi
	# Nuke 4.6 support if it's missing from upstream
	[ -f $(XBUILD)/$(SUITE)/meta/recipes-devtools/gcc/gcc-crosssdk-initial_4.6.bb ] \
	|| rm -f $(XBUILD)/$(SUITE)/meta-linaro/recipes-devtools/gcc/*-4.6*.bb

# Consider downloading the mirror using something like:
fill-downloads:
	wget -N -r -l 1 -nd -P $(XBUILD)/downloads $(MIRROR)

# Push the downloads directory up to the mirror
push-mirror:
	rsync -rlti \
	--exclude="*-linaro-*" \
	--exclude="*.lock" --exclude="*.done" --exclude=git2 --exclude=svn \
	downloads/ \
	builds.linaro.org:/srv/builds.linaro.org/www/toolchain/misc/mirror/openembedded

$(B)/%.tar:
	make -f $(LIBDIR)/fetch.mk fetch F=$@
