# A top-level Makefile that runs steps in a child Makefile and updates
# the stamps.
#

# Prefix of the tarball and directory name made during archive.
# Generally the same as the suite, except for GCC where it includes
# the version
ARCHIVE_BASE ?= $(SUITE)

# Always build
TARGETS += build

ifeq (,$(filter noarchive,$(BUILD_OPTIONS)))
TARGETS += archive
endif

ifeq (,$(filter nobench,$(BUILD_OPTIONS)))
TARGETS += run
endif

ifeq (,$(filter nocheck,$(BUILD_OPTIONS)))
TARGETS += testsuite testsuite-send
endif

BUILD_DIR ?= $(VBUILD)/build

CHILD = $(LIBDIR)/$(SUITE).mk
XCHECK_FAILED = touch $(STEP)-failed.txt

# Compress but don't use a crazy amount of memory.  Compression often
# runs in parallel due to the following xargs
XZ_FLAGS = -9fk -M 300M
XARGS_FLAGS = -P $(NPROCESSORS) -r -L 1

all: $(B)/$(SUITE)-$(SNAME)publish.stamp

# publish: take the outputs including tarballs and log files, put in
# the results directory, and push to the build results host
$(B)/$(SUITE)-$(SNAME)publish.stamp: $(TARGETS:%=$(B)/$(SUITE)-$(SNAME)%.stamp)
	$(call PING_STEP,publish)
	mkdir -p $(RESULTS)/logs/$(SUFFIX)
	$(MAKE) -f $(CHILD) publish
	touch $@

# archive: create a tarball of the installed binary
$(B)/$(SUITE)-$(SNAME)archive.stamp: $(B)/$(SUITE)-$(SNAME)install.stamp
	$(call PING_STEP,archive)
	$(TIME) -o $(@:%.stamp=%.time) $(MAKE) -f $(CHILD) archive
	$(MAKE) -f $(CHILD) post-archive
	touch $@

# install: install the just-built binary
$(B)/$(SUITE)-$(SNAME)install.stamp: $(B)/$(SUITE)-$(SNAME)build.stamp
	$(call PING_STEP,install)
	$(MAKE) -f $(CHILD) install STEP=$(@:%.stamp=%) || $(CHECK_FAILED)
	touch $@

# run: gather the size of everything and run the benchmarks
$(B)/$(SUITE)-$(SNAME)run.stamp: $(B)/$(SUITE)-$(SNAME)build.stamp $(B)/$(SUITE)-$(SNAME)install.stamp
	$(call PING_STEP,run)
	$(MAKE) -f $(CHILD) get-sizes STEP=$(@:%.stamp=%)
	$(MAKE) -f $(CHILD) run STEP=$(@:%.stamp=%)
	touch $@

# testsuite: run the testsuite, normally by running 'make check'
$(B)/$(SUITE)-$(SNAME)testsuite.stamp: $(B)/$(SUITE)-$(SNAME)build.stamp
	$(call PING_STEP,testsuite)
	$(MAKE) -f $(CHILD) testsuite STEP=$(@:%.stamp=%)
	touch $@

# testsuite-send: send testsuite results to a list
$(B)/$(SUITE)-$(SNAME)testsuite-send.stamp: $(B)/$(SUITE)-$(SNAME)testsuite.stamp
	$(call PING_STEP,testsuite-send)
	-$(MAKE) -f $(CHILD) testsuite-send STEP=$(@:%.stamp=%)
	touch $@

# build: build the already configured suite
$(B)/$(SUITE)-$(SNAME)build.stamp: $(B)/$(SUITE)-$(SNAME)configure.stamp
	$(call PING_STEP,build)
	[ -z $(BUILD_TMP) ] || mkdir -p $(BUILD_TMP)
	TMP=$(BUILD_TMP) $(TIME) -o $(@:%.stamp=%.time) $(MAKE) -f $(CHILD) build STEP=$(@:%.stamp=%) || $(CHECK_FAILED)
	touch $@

# configure: configure the suite for building
$(B)/$(SUITE)-$(SNAME)configure.stamp: $(B)/$(SUITE)-$(SNAME)extract.stamp
	$(call PING_STEP,configure)
	mkdir -p $(VBUILD)/build
	$(MAKE) -f $(CHILD) configure STEP=$(@:%.stamp=%)
	touch $@

# extract: extract this particular instance of the source code.  May
# be covered by extract-top.  Extracts into $(VBUILD)
$(B)/$(SUITE)-$(SNAME)extract.stamp: $(B)/$(SUITE)-extract-top.stamp
	$(call PING_STEP,extract)
	mkdir -p $(LBUILD) $(VBUILD)
	$(MAKE) -f $(CHILD) extract STEP=$(@:%.stamp=%)
	touch $@

# extract-top: extract the source code common to all builds.
# Generally extracts into $(LBUILD)
$(B)/$(SUITE)-extract-top.stamp:
	$(call PING_STEP,extract-top)
	mkdir -p $(LBUILD)
	$(MAKE) -f $(CHILD) extract-top STEP=$(@:%.stamp=%)
	touch $@

.PRECIOUS:: \
	$(B)/$(SUITE)-$(SNAME)publish.stamp \
	$(B)/$(SUITE)-$(SNAME)archive.stamp \
	$(B)/$(SUITE)-$(SNAME)install.stamp \
	$(B)/$(SUITE)-$(SNAME)run.stamp \
	$(B)/$(SUITE)-$(SNAME)testsuite.stamp \
	$(B)/$(SUITE)-$(SNAME)testsuite-send.stamp \
	$(B)/$(SUITE)-$(SNAME)build.stamp \
	$(B)/$(SUITE)-$(SNAME)configure.stamp \
	$(B)/$(SUITE)-$(SNAME)extract.stamp \
	$(B)/$(SUITE)-extract-top.stamp

# Default versions of the rules
get-sizes:
	ALL="$(shell find -L $(VBUILD)/install/ -executable -and -type f -exec file {} \; | grep ELF | awk -F: '{print $$1;}' | sort)"; \
	LIBS="$(shell find -L $(VBUILD)/install/ -type f -name '*.a' | sort)"; \
	strip $$ALL; \
	size $$ALL > $(STEP).txt; \
	echo du: >> $(STEP).txt; \
	du -ab $$ALL $$LIBS >> $(STEP).txt

# xz -2 is a good mix between size and speed on a GCC binary.  Gives
# 15 % bigger than the best for 56 s/MB.
# Rename as adding instead of copying.
archive:
	XZ_OPT=-2 \
	tar cJf $(B)/$(ARCHIVE_BASE)-$(SNAME)$(SUFFIX).tar.xz \
	-C $(VBUILD)/install \
	--transform "s,^\./,$(ARCHIVE_BASE)-$(SNAME)$(SUFFIX)/," \
	.

# Toast the old install directory first as gcc complains on overwriting
install:
	rm -rf $(VBUILD)/install
	PATH=$(TOP)/$(VBUILD)/prebuilt/bin:$(TOP)/$(LBUILD)/prebuilt/bin:$$PATH \
	make -C $(BUILD_DIR) install >> $(STEP).txt 2>&1

testsuite:
	$(TIME) -o $(STEP).time make -ks $(PARALLEL) -C $(BUILD_DIR) check > $(STEP).txt 2>&1 || $(XCHECK_FAILED)

testsuite-send:
	true

build:
	PATH=$(TOP)/$(VBUILD)/prebuilt/bin:$(TOP)/$(LBUILD)/prebuilt/bin:$$PATH \
	make -k $(PARALLEL) -C $(BUILD_DIR) $(BUILDTARGET) $(BUILDFLAGS) > $(STEP).txt 2>&1

publish:
	mkdir -p $(RESULTS)/logs/$(SUFFIX)
	# Pull across all DejaGnu test summaries
	find $(BUILD_DIR) -name "*.log" -not -name config.log | xargs $(XARGS_FLAGS) xz $(XZ_FLAGS)
	find $(BUILD_DIR) -name "*.sum" | xargs $(XARGS_FLAGS) xz $(XZ_FLAGS)
	# Pull across gcc and gdb style build/product/testsuite/ logs
	-cp -fa $(BUILD_DIR)/*/*/{,*/}*.{log,sum}.xz $(RESULTS)/logs/$(SUFFIX)
	# Put everything else into a tarball
	-tar cJf /tmp/testsuite-logs.tar.xz \
	`find $(BUILD_DIR) -name "*.log" -not -name config.log` \
	`find $(BUILD_DIR) -name "*.sum"` && \
	mv /tmp/testsuite-logs.tar.xz $(RESULTS)/logs/$(SUFFIX)
	# Grab all config logs
	-tar caf /tmp/config-logs.tar.xz `find $(BUILD_DIR) -name "config*.log"` && \
	mv /tmp/config-logs.tar.xz $(RESULTS)/logs/$(SUFFIX)
	make -f $(LIBDIR)/publish.mk SUITE=$(ARCHIVE_BASE) PUBLISH_SUFFIX=$(PUBLISH_SUFFIX)

# May be covered by extract
extract-top:
	true

# May be covered by extract-top
extract:
	true

# May not need configuring
configure:
	true

# May not run any benchmark
run:
	true

# Normally does nothing.  Useful for pushing before a long test run
post-archive:
	true

.PHONY: publish archive install run testsuite build configure extract-top extract post-archive
