#!/usr/bin/python
"""Record the difference between the test results of this build and
it's ancestor or predecessor.
"""

import sys
import re
import os
import pprint
import subprocess
import logging

import findancestor
import findlogs

TEMPLATE = """Difference in testsuite results between:
 %(current)s build %(buildid)s
and the one before it:
 %(other)s build %(otherbuildid)s

------
%(diff)s"""

def run(allfiles, snapshots, builds):
    # Make the search a bit easier by only keeping the .txt files
    filtered = [x for x in allfiles if '.txt' in x[-1] or '.sum' in x[-1]]

    # And cache by directory
    directories = {}

    for f in filtered:
        directory = '/'.join(f[:-1])
        directories.setdefault(directory, []).append(f)

    # Find all builds that have finished
    for build in [x for x in filtered if x[-1] == 'finished.txt']:
        match = '/'.join(build[:-1])
        # Find all of the files in the same directory
        siblings = directories[match]

        all = ' '.join(x[-1] for x in siblings)

        # Are there any .sum style results?
        if '.sum' not in all:
            continue

        # See if the diff already exists
        if ' testsuite-diff.txt' in all:
            continue

        # Find the ancestor or predecessor
        current = build[1]
        other = findancestor.find(allfiles, current, snapshots)

        if not other:
            logging.info('No known ancestor to %s' % current)
            continue

        # Find the corresponding build
        logs = ['/'.join(x) for x in siblings if '.sum' in x[-1]]
        assert logs
        log = logs[-1]

        try:
            otherlogs = findlogs.find(allfiles, log, other)
        except Exception, ex:
            logging.info('Error %s while finding logs on %s vs %s' % (ex, log, other))
            otherlogs = None
            
        if not otherlogs:
            continue

        other, otherbuildid = otherlogs
        buildid = build[-2]

        try:
            logging.debug('Generating a diff between predecessor %s/%s and %s' % (other, otherbuildid, os.path.dirname(log)))

            diff = subprocess.check_output([
                    './difftests.sh',
                    '%s/%s/logs/%s' % (builds, other, otherbuildid),
                    os.path.join(builds, os.path.dirname(log))
                    ])

        except Exception, ex:
            logging.error(ex)

        if diff == None:
            continue

        with open('%s/%s/testsuite-diff.txt' % (builds, match), 'w') as f:
            print >> f, TEMPLATE % locals()

def main():
#    logging.basicConfig(level=logging.DEBUG)
    allfiles, snapshots, builds = sys.argv[1:]

    with open(allfiles) as f:
        allf = [x.strip().split(os.sep) for x in f.readlines()]

    run(allf, snapshots, builds)

if __name__ == '__main__':
    main()

