#!/bin/sh
# Update a branch and snapshot the latest revision
# Can also snapshot past revisions and add a per-user
# suffix
#
# Example: export-bzr.sh ~/repos/gcc-linaro/4.6 gcc-linaro-4.6 [revno [suffix [basedir]]]
#
set -e

dir=`mktemp -d`
trap "rm -rf $dir" EXIT

# Directory this script runs from
here=`dirname $0`

# Suffix for manual pulls
SUFFIX=${4:-}

# Pull a branch and tarball it up
branch=$1
head=$2

latest=`bzr revno $branch`
revno=${3:-$latest}

base=${5:-$branch}

snapshots=$HOME/snapshots
stem=$head+bzr$revno$SUFFIX
tar=/tmp/$stem.tar
meta=/tmp/$stem.txt
xdelta=$tar.xdelta3.xz

if [ -f $snapshots/`basename $xdelta` ]; then
    exit 0
fi

closest=`$here/closest.py $tar $snapshots/base/*.tar`

mkdir -p $snapshots

# Record some meta data about the branch
echo \# Branch summary > $meta
echo name: $stem >> $meta
echo directory: $branch >> $meta
echo branch: `bzr info $branch | grep parent | awk '{print $3;}'` >> $meta
echo mainline-branch: $base >> $meta
echo revno: $revno >> $meta
echo revision: `bzr revision-info -d $branch -r $revno` >> $meta
echo tip: `bzr revision-info -d $branch` >> $meta
echo ancestor: `bzr revision-info -d $branch -r ancestor:$base` >> $meta
echo closest: `basename $closest` >> $meta
echo xdelta3: `basename $xdelta` >> $meta

echo Exporting and stamping $revno
bzr export -r $revno $dir/$head $branch
REVNO=$revno $here/stamp_branch.sh $branch $dir/$head

# Create the tarball
echo Creating $tar
(cd $dir && find $head -print0 | sort -z) | tar caf $tar -C $dir --no-recursion --null -T -

# Create the delta
echo Making the delta relative to $closest
xdelta3 -fe -s $closest $tar $tar.xdelta3
rm -f $tar
xz -f $tar.xdelta3

mv -f $tar.xdelta3.xz $meta $snapshots

echo New snapshot `basename $tar` created
echo stem: $stem
