#!/usr/bin/python
"""Given a build name, find the ancestor or the last working build of
that series.

Examples:
 findancestor gcc-linaro-4.5+bzr99502~ams~merge-20110413 -> the ancestor
 findancestor gcc-linaro-4.5+bzr99510 -> probably gcc-linaro-4.5+bzr99509
 findancestor gcc-4.6+svn174684 -> the build from last week
"""

import sys
import os.path
import pprint
import re
import pdb
import collections

def parse(name):
    """Parse the meta data file which is a series of name: value pairs"""
    values = {}

    with open(name) as f:
        lines = [x.strip() for x in f.readlines()]

        for line in lines:
            if not line:
                continue

            if line.startswith('#'):
                continue

            name, value = [x.strip() for x in line.split(':', 1)]
            values[name] = value

    return values

def revkey(record):
    if record.date:
        return int(record.date.replace('~', ''))
    else:
        return int(record.revision)

def make_splitter():
    """Splits a build name like gcc-4.5+svn1234,
    gcc-linaro-4.6+bzr1234, and gdb-7.3~20110606+gitabcdef into the
    product, series, optional date, version control system, and
    revision.

    >>> make_splitter().match('gcc-4.5~svn12345').groups()
    ('gcc-', '4.5', None, '~svn', '12345')
    >>> make_splitter().match('gcc-linaro-4.6+bzr4567').groups()
    ('gcc-linaro-', '4.6', None, '+bzr', '4567')
    >>> make_splitter().match('gdb-7.3~20110606+git057a947').groups()
    ('gdb-', '7.3', '~20110606', '+git', '057a947')
    """
    return re.compile('([\D\-]+)([\d\.]+)([+~]\d+)?([+~][a-z]{3})(\w+)$')

def make_release_splitter():
    """Splits a release name like gcc-linaro-4.5-2011.03-0 into
    product, series, and release.

    >>> make_release_splitter().match('gcc-linaro-4.5-2011.03-0').groups()
    ('gcc-linaro-', '4.5', '-2011.03-0')
    """
    return re.compile('([\D\-]+)([\d\.]+)(-[\d.]+-\d+)$')

def find_via_meta(name, snapshots):
    # See if we can find the meta data and extract the ancestor
    metaname = os.path.join(snapshots, '%s.txt' % name)

    try:
        meta = parse(metaname)
        revision, prev, mainline = meta['revision'], meta['ancestor'], meta['mainline-branch']

        if revision != prev:
            return '%s+bzr%s' % (mainline, prev.split()[0])
    except Exception, ex:
        pass

    return ''

def find_via_build(allfiles, name):
    # Don't know the ancestor.  Find the previous build
    Record = collections.namedtuple('Record', 'product series date vcs revision')

    splitter = make_splitter()
    builds = set([x[1] for x in allfiles if x[-1] == 'finished.txt'])
    builds = [splitter.match(x) for x in sorted(builds)]
    builds = [Record(*x.groups()) for x in builds if x]

    match = splitter.match(name)

    if match:
        this = Record(*match.groups())

        # Grab the builds that match
        matches = [x for x in builds
                   if x.product == this.product
                   and x.series == this.series
                   and x.vcs == this.vcs]

        # Sort them by revno
        matches.sort(key=revkey)

        # Find this build and the one before
        revnos = [x.revision for x in matches]

        if this.revision in revnos:
            idx = revnos.index(this.revision)

            if idx > 0:
                return ''.join(x for x in matches[idx-1] if x)

    return ''

def find_via_release(allfiles, name):
    Record = collections.namedtuple('Record', 'product series release')

    # Don't know the ancestor.  Find the previous release
    splitter = make_release_splitter()
    builds = set([x[1] for x in allfiles if x[-1] == 'finished.txt'])
    builds = [splitter.match(x) for x in sorted(builds)]
    builds = [Record(*x.groups()) for x in builds if x]

    match = splitter.match(name)

    if match:
        this = Record(*match.groups())

        # Grab the builds that match
        matches = [x for x in builds
                   if x.product == this.product
                   and x.series == this.series]

        # Sort them by release
        matches.sort(key=lambda x: x.release)

        # Find this build and the one before
        releases = [x.release for x in matches]

        if this.release in releases:
            idx = releases.index(this.release)

            if idx > 0:
                return ''.join(x for x in matches[idx-1] if x)

    return ''

def find(allfiles, name, snapshots):
    ancestor = find_via_meta(name, snapshots)

    if ancestor:
        return ancestor

    ancestor = find_via_build(allfiles, name)

    if ancestor:
        return ancestor

    ancestor = find_via_release(allfiles, name)

    if ancestor:
        return ancestor

    return ''

def main():
    allfiles, name = sys.argv[1:]

    with open(allfiles) as f:
        allf = [x.strip().split(os.sep) for x in f.readlines()]

    print find(allf, name, '')

def test():
    import doctest
    doctest.testmod()

if __name__ == '__main__':
    main()
