#!/usr/bin/python
"""Given a log file from a build and the build's ancestor, find the
corresponding log file for the ancestor.

Used to find the GCC testsuite results from the ancestor so that they
can be compared.

Example:
 findlogs.py build/all-files.txt build/gcc-4.6+svn173722/logs/armv7l-maverick-cbuild114-ursa4-cortexa9r1/gcc.sum.xz gcc-4.6+svn173209
 "gcc-4.6+svn173209 armv7l-maverick-cbuild113-ursa3-cortexa9r1"

"""

import sys
import os.path
import pprint
import re
import pdb

def find(allfiles, log, ancestor):
    log = log.split('/')
    arch, osname, cbuild, host, config = log[-2].split('-')

    # Tidy up the ancestor
    match = re.match('lp\:([^/]+)(/.+)?([+~]\w+\d+)', ancestor)

    if match:
        branch, series, revision = match.groups()

        if series:
            series = series[1:]
        elif branch == 'gcc-linaro':
            # gcc-linaro is the same as gcc-linaro/4.5
            series = '4.5'
        else:
            assert False, 'Unrecognised development focus for %s' % branch
    
        ancestor = '%s-%s%s' % (branch, series, revision)

    # Select those that could be a log
    files = [x[-4:] for x in allfiles if len(x) >= 4]

    # Split out the build ID
    for f in files:
        f[-2] = (f[-2].split('-') + ['']*5)[:5]

    # Select results from the ancestor
    files = [x for x in files if x[-4] == ancestor]
    # ...and those that match the build log
    files = [x for x in files if x[-1] == log[-1]]
    # ...and that have the same architecture and config
    files = [x for x in files if x[-2][0] == arch and x[-2][-1] == config]

    # Sort by cbuild number
    files.sort(key=lambda x: int(x[-2][2].replace('cbuild', '')))

    if files:
        best = files[-1]
        return (best[0], '-'.join(best[2]))
    else:
        return None

def main():
    allfiles, log, ancestor = sys.argv[1:]

    with open(allfiles) as f:
        allf = [x.strip().split(os.sep) for x in f.readlines()]

    best = find(allf, log, ancestor)

    if best:
        print ' '.join(best)

if __name__ == '__main__':
    main()
